/**
Die Klasse User dient zur Speicherung von Usern
in einem Online-Shop.

- author: Peter Brichzin
- version: 23.5.24
 */

class User
{
    /** id - Schlüssel dieses Datenelements */
    private var id: Int
    /** Nachname  */
    private var nachname: String
    /** Vorname  */
    private var vorname: String

    /**
    Konstruktor für Objekte der Klasse User
    - parameters:
        - idNeu: Identifikationsnummer
        - nachnameNeu: Nachname
        - vornameNeu: Vorname
    */
    init(idNeu: Int, nachnameNeu: String, vornameNeu: String)
    {
        id = idNeu
        nachname = nachnameNeu
        vorname = vornameNeu
    }

    /**
    Gibt Information über das Datenelement zu Kontrollzwecken
    auf das Terminalfenster aus.
    */
    func InformationAusgeben()
    {
        print("Der User " + vorname + " " + nachname + " hat die Id \(id).");
    }

    /**
    ergleicht zwei Datenelemente auf Gleichheit.
    - parameters:
        - dvergleich: Datenelement mit dem das Objekt verglichen wird.
    - returns: true, wenn die beiden Datenelemente gleichen Schlüssel haben.
    */
    func IstGleich(dVergleich: User) -> Bool
    {
        if nachname == dVergleich.NachnameGeben()
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Datenelemente bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichselement: Datenelement mit dem das Objekt verglichen wird.
        - returns: true, wenn das aktuelle Element einen größeren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func IstGrößerAls(dVergleich: User) -> Bool
    {
        if nachname > dVergleich.NachnameGeben()
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, auf Gleichheit.
    - parameters:
        - vergleichsSchlüssel: Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn die beiden Schlüssel gleich sind.
    */
    func SchlüsselIstGleich(vergleichsSchlüssel: String) -> Bool
    {
        if nachname == vergleichsSchlüssel
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichsSchlüssel: Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn das aktuelle Element einen grö&szlig;eren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func SchlüsselIstGrößerAls(vergleichsSchlüssel: String) -> Bool
    {
        if nachname > vergleichsSchlüssel
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Geben-Methode zum Attribut nachname
    - returns: Nachname
    */
    func NachnameGeben() -> String
    {
        return nachname
    }

    /**
    Geben-Methode zum Attribut vorname
    - returns: Vorname
    */
    func VornameGeben() -> String
    {
        return vorname
    }
}

/**
Klasse Knoten zur Verwaltung von Usern als Datenelement
innerhalb eines geordneten Binärbaums.
(Implementierung ohne Entwurfsmuster Kompositum)

- author: Peter Brichzin
- version: 23.5.24
*/
class Knoten
{

    /** Referenz auf das Datenelement */
    private var daten: User

    /** Referenz auf den linken und rechten Nachfolger */
    private var linkerNachfolger: Knoten?

    /**
    Konstruktor für Objekte der Klasse Knoten ohne linken und rechten Nachfolger,
    d.h. die entsprechenden Referenzattribute haben den "Wert" nil.
    - parameters: datenNeu: Referenz auf das Datenelement, das vom Knoten verwaltet wird.
    */
    init(datenNeu: User)
    {
        daten = datenNeu
        linkerNachfolger = nil
    }

    /**
    Fügt einen Usern als Datenelement, falls sein Schlüssel noch nicht vorhanden ist,
    in die bestehende Datenstruktur ein. D.h. die Eigenschaft geordneter Binärbaum
    bleibt erhalten.
    - parameters:
        - datenNeu: einzufügender User als Datenelement
    */
    func Einfügen(datenNeu: User)
    {
        // Falls es ein Datenelement mit dem gleichen Schlüssel schon gibt: Einfügen abbrechen.
        if daten.IstGleich(dVergleich: datenNeu)
        {
            print("Die Daten existieren schon!")
        }
        // Abfrage, ob im rechten oder linken Teilbaum eingefügt werden muss
        else
        {
            if daten.IstGrößerAls(dVergleich: datenNeu)
            {
                // wenn es einen linken Nachfolger gibt,
                // dannn wird das Datenelement im linkem Teilbaum eingefügt,
                // sonst wird ein Knoten mit einer Referenz auf das Datenelement erzeugt und als linker Nachfolger zugewiesen
                if linkerNachfolger != nil
                {
                    linkerNachfolger!.Einfügen(datenNeu: datenNeu)
                }
                else
                {
                    linkerNachfolger = Knoten(datenNeu: datenNeu)
                }
            }
            else
            {
                // wenn es einen rechten Nachfolger gibt,
                // dannn wird das Datenelement im rechten Teilbaum eingefügt,
                // sonst wird ein Knoten mit einer Referenz auf das Datenelement erzeugt und als rechter Nachfolger zugewiesen
                if rechterNachfolger != nil
                {
                    rechterNachfolger!.Einfügen(datenNeu: datenNeu)
                }
                // sonst Datenelement im rechten Teilbaum einfügen
                else
                {
                    rechterNachfolger = Knoten(datenNeu: datenNeu)
                }
            }
        }
    }

    /**
    Geben-Methode zu den Daten
    - returns: Referenz auf die Daten
    */
    func DatenGeben() -> User
    {
        return daten
    }

    /**
    Geben-Methode zum linken Nachfolger
    - returns: Referenz auf LinkerNachfolger
    */
    func LinkerNachfolgerGeben() -> Knoten?
    {
        return linkerNachfolger
    }

    /**
    Geben-Methode zum rechten Nachfolger
    - returns: Referenz auf RechterNachfolger
    */
    func RechterNachfolgerGeben() -> Knoten?
    {
        return rechterNachfolger
    }
}

/**
Die Klasse Binbaum ist die Grundstruktur eines geordneten Binärbaums
und hier zur Verwaltung von Usern eingesetzt wird.
Die Methoden die ein Objekt dieser Klasse anbietet, leiten den Aufruf an den
Wurzelknoten weiter, wenn der Baum nicht leer ist.
(Implementierung ohne Entwurfsmuster Kompositum)

- author: Peter Brichzin
- version: 24.5.24
*/
 class BinBaum
{
    /** Die Wurzel des Baums */
     private var wurzel: Knoten?

    /**
    Konstruktor für Objekte der Klasse Binbaum:
    Ein leerer Baum wird erzeugt.
    */
    init()
    {
        wurzel = nil
    }

    /**
    Fügt ein Datenelement sortiert in den geordneten Binärbaum ein,
    d.h. die Eigenschaft, dass der Baum geordnet ist bleibt bei jedem
    Einfügevorgang erhalten.
    - parameters:
        - datenNeu: neues Datenelement
    */
     func Einfügen(datenNeu: User)
    {
        if wurzel != nil
        {
            wurzel!.Einfügen(datenNeu: datenNeu)
        }
        else
        {
            wurzel = Knoten(datenNeu: datenNeu)
        }
    }
}

let user1 = User(idNeu: 5781, nachnameNeu: "Geber", vornameNeu: "Ann")
let user2 = User(idNeu: 4221, nachnameNeu: "Tross", vornameNeu: "Albert")
let user3 = User(idNeu: 6345, nachnameNeu: "Fall", vornameNeu: "Klara")
let user4 = User(idNeu: 5799, nachnameNeu: "Ter", vornameNeu: "Lee")
let user5 = User(idNeu: 6300, nachnameNeu: "Sehr", vornameNeu: "Kai")
let user6 = User(idNeu: 5888, nachnameNeu: "Pfahl", vornameNeu: "Marta")
let  binBaum1 = BinBaum()
binBaum1.Einfügen(datenNeu: user1)
binBaum1.Einfügen(datenNeu: user2)
binBaum1.Einfügen(datenNeu: user3)
binBaum1.Einfügen(datenNeu: user4)
