/**
Die Klasse Woerterbucheintrag dient zur Speicherung von
Wörterbucheinträgen für ein Englisch-Deutsches
Wörterbuch. Dabei kann jedem englischen Wort mehrere
deutsche Bedeutungen zugeordnet werden, wobei alle Bedeutungen in
einer einzigen Zeichenkette Bedeutung gespeichert werden.

- author: Peter Brichzin
- version: 23.05.24
*/
class Woerterbucheintrag
{
    /** Englisches Wort;  Schlüssel dieses Datenelements */
    private var wort: String
    /** Deutsche Bedeutung; mehrere Bedeutungen werden als eine Zeichenkette zusammengefasst */
    private var bedeutung: String

    /**
    Konstruktor für Objekte der Klasse Woerterbucheintrag
    - parameters:
        - wortNeu: das englische Wort.
        - bedeutungNeu die deutsche(n) Bedeutung(en).
    */
    init(wortNeu: String, bedeutungNeu: String)
    {
        wort = wortNeu
        bedeutung = bedeutungNeu
    }

    /**
    Gibt Information über das Datenelement zu Kontrollzwecken
    auf das Terminalfenster aus.
    */
    func InformationAusgeben()
    {
        print(wort + ": " + bedeutung)
    }

    /**
    Vergleicht zwei Datenelemente auf Gleichheit.
    - parameters:
        - dvergleich: Datenelement mit dem das Objekt verglichen wird.
    - returns: true, wenn die beiden Datenelemente gleichen Schlüssel haben.
    */
    func IstGleich(dvergleich: Woerterbucheintrag) -> Bool
    {
        if wort == dvergleich.WortGeben()
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Datenelemente bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichselement: Datenelement mit dem das Objekt verglichen wird.
    - returns: true, wenn das aktuelle Element einen größeren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func IstGrößerAls(dvergleich: Woerterbucheintrag) -> Bool
    {
        if wort > dvergleich.WortGeben()
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, auf Gleichheit.
    - parameters:
        - vergleichsSchlüssel: Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn die beiden Schlüssel gleich sind.
    */
    func SchlüsselIstGleich(vergleichsSchlüssel: String) -> Bool
    {
        return wort == vergleichsSchlüssel
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichsSchlüssel Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn das aktuelle Element einen größeren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func SchlüsselIstGrößerAls(vergleichsSchlüssel: String) -> Bool
    {
        if wort > vergleichsSchlüssel
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Vergleicht zwei Schlüssel, die als Zeichenketten vorliegen, bezüglich der Ordnungsrelation.
    - parameters:
        - vergleichsSchüssel Schlüssel  mit dem der Schlüssel des Objekt verglichen wird.
    - returns: true, wenn das aktuelle Element einen kleineren Schlüssel hat, als das angegebene Vergleichselement.
    */
    func SchlüsselIstKleinerAls(vergleichsSchlüssel: String) -> Bool
    {
        if wort < vergleichsSchlüssel
        {
            return true
        }
        else
        {
            return false
        }
    }

    /**
    Gibt den Schlüssel des Datenelements als String aus, auch wenn der Datentyp vom
    String abweicht. Diese Methode wird beim Entfernen eines Knotens benötigt.
    - returns: Schlüssel als String.
     */
    func SchlüsselAlsStringGeben() -> String
    {
        return wort
    }

    /**
    Geben-Methode zum Attribut wort
    - returns: das englische Wort
    */
    func WortGeben() -> String
    {
        return wort
    }

    /**
    Geben-Methode zum Attribut bedeutung
    - returns: bedeutung: die deutsche(n) Bedeutung(en) als eine Zeichenkette
    */
    func BedeutungGeben() -> String
    {
        return bedeutung
    }

    /**
    Setzen-Methode zum Attribut bedeutung
    - parameters:
        - bedeutungNeu: neue Bedeutung für das englische Wort
    */
    func BedeutungSetzen(bedeutungNeu: String)
    {
        bedeutung = bedeutungNeu
    }
}

